/* Time Base Facility API.  API implementation.
 *
 * Copyright IBM Corp. 2014
 *
 * The MIT License (MIT)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * Contributors:
 *     IBM Corporation, Adhemerval Zanella - Initial implementation.
 *     IBM Corporation, Rajalakshmi S - Initial implementation.
 */

#include <stdio.h>
#include <inttypes.h>
#include <paf/tb.h>

int
main ()
{
  int i, j;
  timing load_time, start, stop, sum = 0;

  for (i = 0; i < 3; i++)
    {
      start = paf_timing_now ();
      for (j = 0; j < 1000; j++);
      stop = paf_timing_now ();
      load_time = paf_timing_diff (start, stop);
      paf_timing_accum (&sum, load_time);
      printf ("Time taken by the for loop is %" PRIu64 " ticks\n", load_time);
    }
  printf ("Total time taken is %" PRIu64 " ticks\n",sum);
  return 0;
}
